// Copyright (c) 2002 Cathay Networks, Inc.  
// see LICENSE file for details of VOCAL license

/******************* configuration setting stuff ********************/

#include <cassert>
#include <cstdio>
#include <cstring>
#include <errno.h>
#include <iostream.h>

#ifdef WIN32

#include <winsock2.h>
#include <stdlib.h>
#include <io.h>

#else

#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>

#endif


#include "config.h"
#include "network.h"
#include "probe.h"
#include "peer.h"
#include "rib.h"

Setting setting;

/// take a string and convert it to a int and overwrite the value of v
void
setValue( int* v, char* s )
{
   assert( v );
   assert( s );
    
   char* endptr;
   long int val = strtol( s, &endptr, 0);
   if ( endptr != NULL )
   {
      if ( *endptr != '\0' )
      {
      }
   }
   *v = val;
}


void 
parseCommandLine(int argc, char* argv[] )
{
    
   setting.reqPort=5060;
   setting.respPort=5061;
   setting.peerListenPort=7000;
   setting.sinkPort=0;
   setting.cps=0;
    
   setting.httpStatsPort=8080;
   setting.printStatsTime=5;
   setting.probePort=5062;
   setting.stickyTableSize=2048;
   setting.stickyExpireTime = 30*60; // this is in seconds 
   setting.checkMaxHop=false;
   setting.checkProxyRequire=false;
   setting.loopDetect=false;
   setting.verbose=false;

   setting.testRandomRib=false;
   setting.supportNat = false;

   strcpy(setting.useStickyType,"Call-ID");
    
   // TODO add code to guesse host name of this proxy
   //strcpy(setting.proxyName,"localhost");
   gethostname(setting.proxyName,512);
   cerr << "Hostname is: " << setting.proxyName <<endl;
    
#ifdef WIN32
   int cur =1;
   while ( cur < argc )
   {
      assert( argv[cur] );

      if ( argv[cur][0] == '-' )
      {
         // got an option
         if ( cur+1 >= argc )
         {
            cerr << "missing option after " << argv[cur] << endl;
            cur++;
            exit(1);
         }
         char* optarg = argv[cur+1];
         assert( optarg );
            
         if ( !strcmp(argv[cur],"-reqPort") ) setValue( &(setting.reqPort), optarg );
         if ( !strcmp(argv[cur],"-peerListenPort") ) setValue( &(setting.peerListenPort), optarg );
         if ( !strcmp(argv[cur],"-peer") )
         {
            // peer 
            //clog << "Peer with " << optarg << endl;
            if ( numPeers+1 >= maxNumPeers )
            {
               cerr << "Max peers exceeded - ignoring " << optarg << endl;
            }
            else
            {
               int p = numPeers++;
               peer[p].id=0;
               strncpy(peer[p].peerName,optarg,512); peer[p].peerName[512-1]='0';
               peer[p].fd=INVALID_SOCKET;
            }
         }

         cur += 2;
      }
      else
      {
         //clog << "proxy to " << argv[cur] << endl;
            
         int ip;
         int port;
            
         parseHostName( argv[cur] , &ip , &port );
            
         addProxy(ip,port,1/*priority*/,1/*weight*/,false);
            
         cur++;
      }
   }

#else
    
   static struct option options[] =
      {
         {"loopDetection",0,0,0},
         {"checkProxyRequire",0,0,0},
         {"checkMaxHop",0,0,0},
         {"verbose",0,0,0},
         {"name",1,0,0}, // 5 
         {"reqPort",1,0,0},
         {"respPort",1,0,0},
         {"probePort",1,0,0},
         {"httpStatsPort",1,0,0},
         {"printStatsTime",1,0,0}, 
         {"stickyTableSize",1,0,0}, // 10 
         {"useStickyType",1,0,0},
         {"stickyExpireTime",1,0,0},
         {"peer",1,0,0},
         {"peerListenPort",1,0,0}, 
         {"sinkPort",1,0,0}, // 15 
         {"proxy",1,0,0},
         {"noProbeProxy",1,0,0},
         {"genReq",1,0,0},
         {"genResp",1,0,0}, 
         {"genRespDst",1,0,0}, //20
         {"cps",1,0,0}, 
         {"runTime",1,0,0}, 
         {0, 0, 0, 0}
      };
    
   while (true)
   {
      int index = 0;
        
      int c = getopt_long_only(argc, argv, "",
                               options, &index);
      if (c == -1)
      {
         for ( int i=optind ; i < argc; i++ )
         {
            //clog << "proxy to " << argv[i] << endl;
                
            int ip;
            int port;
                
            parseHostName( argv[i] , &ip , &port );
            addProxy(ip,port,1/*priority*/,1/*weight*/,true);
         }
         break;
      }
        
      if ( c != 0 )
      {
         cerr << "Usage is: - read the code" << endl;
         exit(1);
      }
        
      switch (index)
      {
         case 2: setting.checkMaxHop=true; break;
         case 1: setting.checkProxyRequire=true; break;
         case 0: setting.loopDetect=true; break;
         case 3: setting.verbose=true; break;
            
         case 22: setValue( &(setting.runTime), optarg ); break;
         case 21: setValue( &(setting.cps), optarg ); break;
         case 15: setValue( &(setting.sinkPort), optarg ); break;
         case 5:  setValue( &(setting.reqPort), optarg ); break;
         case 6:  setValue( &(setting.respPort), optarg ); break;
         case 14: setValue( &(setting.peerListenPort), optarg ); break;
         case 8:  setValue( &(setting.httpStatsPort), optarg ); break;
         case 9:  setValue( &(setting.printStatsTime), optarg ); break;
         case 7:  setValue( &(setting.probePort), optarg ); break;
         case 10: setValue( &(setting.stickyTableSize), optarg ); break;
            
         case 4: 
            strncpy(setting.proxyName,optarg,512); setting.proxyName[512-1]='\0';
            break;
         case 11: 
            strncpy(setting.useStickyType,optarg,64); setting.useStickyType[64-1]='\0';
            break;
            
         case 13:
         {
            // peer 
            //clog << "Peer with " << optarg << endl;
            if ( numPeers+1 >= maxNumPeers )
            {
               cerr << "Max peers exceeded - ignoring " << optarg << endl;
            }
            else
            {
               int p = numPeers++;
               peer[p].id=0;
               strncpy(peer[p].peerName,optarg,512); peer[p].peerName[512-1]='0';
               peer[p].fd=INVALID_SOCKET;
            }
         }
         break;
            
         case 16:
         {
            // proxy  
            clog << "proxy to " << optarg << endl;
                
            int ip;
            int port;
                
            parseHostName( optarg , &ip , &port );
            addProxy(ip,port,1/*priority*/,1/*weight*/,true/*probe*/);
         }
         break;
            
         case 17:
         {
            // proxy  
            clog << "proxy to " << optarg << endl;
                
            int ip;
            int port;
                
            parseHostName( optarg , &ip , &port );
            addProxy(ip,port,1/*priority*/,1/*weight*/,false/*probe*/);
         }
         break;
            
         case 18:
         {
            parseHostName( optarg , &setting.genReqIp , &setting.genReqPort );
         }
         break;
            
         case 19:
         {
            parseHostName( optarg , &setting.genRespIp , &setting.genRespPort );
         }
         break;
            
         case 20:
         {
            parseHostName( optarg , &setting.genRespDstIp , &setting.genRespDstPort );
         }
         break;
            
         default:
            cerr << "Option " << index << " " 
                 << options[index].name << " not implemented" << endl;
      }
   }
#endif 
}

// Local Variables:
// mode:c++
// c-file-style:"bsd"
// c-file-offsets:((case-label . +))
// c-basic-offset:4
// indent-tabs-mode:nil
// End:
