// Copyright (c) 2002 Cathay Networks, Inc.  
// see LICENSE file for details of VOCAL license

#include <cassert>
#include <cstdio>
#include <cstring>
#include <errno.h>
#include <iostream.h>

#ifdef WIN32

#include <winsock2.h>
#include <stdlib.h>
#include <io.h>

#else

#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#endif


#include "config.h"
#include "loadgen.h"
#include "network.h"
#include "peer.h"
#include "probe.h"
#include "rib.h"
#include "sip.h"
#include "stats.h"
#include "time.h"
#include "udp.h"

int 
main(int argc, char* argv[])
{
   assert( sizeof(Int8) == 1 );
   assert( sizeof(Int16) == 2 );
   assert( sizeof(Int32) == 4 );
   assert( sizeof(Int64) == 8 );

   cerr << "Starting " << argv[0] << endl;

   getTime(true); // make sure time is initialzied 

#if 0
   Int64 t = getTime();
   
   t /= oneSecond; // convert to seconds 
   t /= 60*60*24; // convert to days 
   double f = t;
   f /= 365.25; // convert to years 
   f += 1900.0; // add NTP epoc

   cerr << "\t aprox year is ";
   cerr.precision(7);
   cerr << f << endl;
#endif

#ifdef WIN32 
   WORD wVersionRequested = MAKEWORD( 2, 2 );
   WSADATA wsaData;
   int err;

   err = WSAStartup( wVersionRequested, &wsaData );
   if ( err != 0 ) 
   {
      // could not find a usable WinSock DLL
      cerr << "Could not load winsock" << endl;
      assert(0); // is this is failing, try a different version that 2.2, 1.0 or later will likely work 
      exit(1);
   }
    
   /* Confirm that the WinSock DLL supports 2.2.*/
   /* Note that if the DLL supports versions greater    */
   /* than 2.2 in addition to 2.2, it will still return */
   /* 2.2 in wVersion since that is the version we      */
   /* requested.                                        */
    
   if ( LOBYTE( wsaData.wVersion ) != 2 ||
        HIBYTE( wsaData.wVersion ) != 2 ) 
   {
      /* Tell the user that we could not find a usable */
      /* WinSock DLL.                                  */
      WSACleanup( );
      cerr << "Bad winsock verion" << endl;
      assert(0); // is this is failing, try a different version that 2.2, 1.0 or later will likely work 
      exit(1);
   }    
#endif

   parseCommandLine(argc,argv);
    
   // allocate rib
   assert( setting.stickyTableSize > 0 );
   rib = new Rib[setting.stickyTableSize];
   assert( rib );
   ribSize = setting.stickyTableSize;
   for ( int i=0; i<ribSize; i++ )
   {
      rib[i].proxy = -1;
   }
    
   // peer listen file descript 
   int peerListenFd = createPeerListen(setting.peerListenPort);
    
   // open request port
   int reqFd=openRecvPort(setting.reqPort);
    
   // open response port
   int respFd=openRecvPort(setting.respPort);
    
   // open probe port
   int probeFd=openRecvPort(setting.probePort);
    
   // open sink port
   int sinkFd=openRecvPort(setting.sinkPort);
    
   if ( setting.testRandomRib )
   {
      cerr << "Filling sticky chache with pointers to proxy 0" << endl;

      Int64 now = getTime();

      for( int i=0; i<ribSize; i++)
      {
         rib[i].proxy=0;
         rib[i].hash=i;
         rib[i].time=now;
      }
   }

   // set up the timeout time 
   struct timeval tv;

   Int64 startTime = getTime();
    
   // loop processing input messages 
   bool run = true;
   while ( run )
   {
      if ( setting.runTime != 0 )
      {
         if ( getTime() > startTime + oneSecond*setting.runTime )
         {
            run = false;
         }
      }
	
      processPeers();
      processProbes(probeFd);
      processStats();
      processGen(sinkFd);
	
      // TODO - move some of this fd set stuff out so not recomputed every time
        
      // set up the file descriptor set
#ifdef _MSC_VER
# pragma warning( disable : 4018 ) 
#endif
      fd_set fdSet; int fdSetSize;
      FD_ZERO(&fdSet); fdSetSize=0;
      if ( reqFd != INVALID_SOCKET ) 
      { 
         FD_SET(reqFd,&fdSet); 
         fdSetSize = max( reqFd+1, fdSetSize );
      }
      if ( respFd != INVALID_SOCKET ) 
      { 
         FD_SET(respFd,&fdSet); 
         fdSetSize = max( respFd+1, fdSetSize ); 
      }
      if ( probeFd != INVALID_SOCKET ) 
      { 
         FD_SET(probeFd,&fdSet); 
         fdSetSize = max( probeFd+1, fdSetSize );
      }
      if ( sinkFd != INVALID_SOCKET ) 
      { 
         FD_SET(sinkFd,&fdSet); 
         fdSetSize = max( sinkFd+1, fdSetSize );
      }
      if ( peerListenFd != INVALID_SOCKET ) 
      { 
         FD_SET(peerListenFd,&fdSet); 
         fdSetSize = max( peerListenFd+1, fdSetSize );
      }
      for( int i=0; i<numPeers; i++)
      {
         int fd = peer[i].fd;
         if ( fd != INVALID_SOCKET )
         {
            FD_SET(fd,&fdSet); fdSetSize = max( fd+1, fdSetSize );
         }
      }
      if ( fdSetSize == 0 )
      {
         //cerr << "Could not open any sockets" << endl;
      }
#ifdef _MSC_VER
# pragma warning( default : 4018 ) 
#endif

      // set up the timeout for 100 ms - need to reset these each time because linux trashes them
      tv.tv_sec=0;
      tv.tv_usec=10*1000;
        
      // wait for input 
      //cerr << "in select" << endl;
      int  err = select(fdSetSize, &fdSet, NULL, NULL, &tv);
      int e = errno;

      getTime( true ); // set the time to whenever the select returned 
	
      //cerr << "exit select" << endl;;

      if ( err == SOCKET_ERROR )
      {
         switch (e)
         {
#ifdef WIN32
            case WSAEINVAL: 
#endif
            case EINVAL:
            {
               if ( fdSetSize == 0 ) break;
               //clog << "Invalid value error in select" << endl;
            }
            break;
            default:
               // error occured
               cerr << "Error " << e << " " << strerror(e) << " in select" << endl;
               break;
         }
      }
      else if ( err == 0 )
      {
         // timeout occured 
         //clog << "Timeout" << endl;
      }
      else
      {
         //clog << "-----------------------------------------" << endl;
            
         assert( err>0 );
         // data is avialbe on some fd 
         if ( respFd!=INVALID_SOCKET ) if ( FD_ISSET(respFd,&fdSet) )
         {
            handleResponse(respFd,reqFd);
         }
            
         if ( reqFd!=INVALID_SOCKET ) if ( FD_ISSET(reqFd,&fdSet) )
         {
            handleRequest(reqFd);
         }
            
         if ( probeFd!=INVALID_SOCKET ) if ( FD_ISSET(probeFd,&fdSet) )
         {
            handleProbe(probeFd);
         }
            
         if ( sinkFd!=INVALID_SOCKET ) if ( FD_ISSET(sinkFd,&fdSet) )
         {
            handleSink(sinkFd);
         }
            
         if ( peerListenFd!=INVALID_SOCKET ) if ( FD_ISSET(peerListenFd,&fdSet) )
         {
            addPeer( peerListenFd );
            //int peerFd = acceptPeer( peerListenFd );
            //assert( peerFd != INVALID_SOCKET );
         }
            
         for (int p=0; p<numPeers; p++)
         {
            int fd = peer[p].fd;
            if ( fd != INVALID_SOCKET )
            {
               if ( FD_ISSET(fd,&fdSet) )
               {
                  // got some input from a peer 
                  receiveUpdate( fd );
               }
            }
         }
      }
   }
    
   if (reqFd!=INVALID_SOCKET) 
   {
      closesocket( reqFd ); reqFd = INVALID_SOCKET;
   }
    
   if (respFd!=INVALID_SOCKET) 
   {
      closesocket( respFd ); respFd =INVALID_SOCKET;
   }
    
   if (probeFd!=INVALID_SOCKET) 
   {
      closesocket( probeFd ); probeFd=INVALID_SOCKET;
   }
    
   if (sinkFd!=INVALID_SOCKET) 
   {
      closesocket( sinkFd ); sinkFd=INVALID_SOCKET;
   }
    
   if (peerListenFd!=INVALID_SOCKET) 
   {
      closesocket( peerListenFd ); peerListenFd=INVALID_SOCKET; 
   }
    
   for (int p=0; p<numPeers; p++)
   {
      closePeer( p );
   }
    
   processStats( true );
    
   assert(rib);
   delete [] rib; rib=NULL; ribSize=0;
    
   return 0;
}

// Local Variables:
// mode:c++
// c-file-style:"bsd"
// c-file-offsets:((case-label . +))
// c-basic-offset:4
// indent-tabs-mode:nil
// End:
