/******************** Load Gen test stuff ************************/
// Copyright (c) 2002 Cathay Networks, Inc.  
// see LICENSE file for details of VOCAL license


#include <cassert>
#include <cstdio>
#include <cstring>
#include <errno.h>
#include <iostream.h>

#ifdef WIN32

#include <winsock2.h>
#include <stdlib.h>
#include <io.h>

#else

#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>

#endif


#include "loadgen.h"
#include "udp.h"
#include "stats.h"
#include "config.h"
#include "time.h"

/// just eat message to form a sink for load generation 
void
handleSink( Socket fd )
{ 
   char buf[udpMaxMessageLength];
   int len;
   unsigned int srcIp;
   unsigned short srcPort;
    
   if ( getMessage(fd,buf,&len,&srcIp,&srcPort) )
   {
      stats.numSinkMsg++;
      //clog << "Sink received " << len << " bytes" << endl;
   }
   if ( getMessage(fd,buf,&len,&srcIp,&srcPort) )
   {
      stats.numSinkMsg++;
      //clog << "Sink received " << len << " bytes" << endl;
   }
   if ( getMessage(fd,buf,&len,&srcIp,&srcPort) )
   {
      stats.numSinkMsg++;
      //clog << "Sink received " << len << " bytes" << endl;
   }
}


/// generate load stuff 
void
processGen( Socket fd ) // uses the skin fd for sending stuff 
{
   // return if genration is not turned on 
   if ( setting.cps == 0 )
   {
      return;
   }
    
   static Int64 lastTime=0;
   Int64 now=getTime();
   if ( lastTime == 0 ) // special case for first time run 
   {
      lastTime = now;
   }
    
   // return right away if nothing to do
   if ( lastTime + oneSecond/setting.cps > now )
   {
      return;
   }

   if ( fd == INVALID_SOCKET )
   {
      cerr << "sink port needs to be valid for load generation" << endl;
      return;
   }
    
   Int64 elapse = ((now-lastTime)*setting.cps/oneSecond);
   lastTime = now;

   //cerr << "Need to do " << elapse << endl;
   for ( int i=0; i<elapse; i++ )
   {
      //clog << "Do generate mesage " << endl;
      char buf[udpMaxMessageLength];
      char* msgVec[1];
      int msgLen[1];
      msgVec[0] = buf;
    
      static int count=0;
      sprintf(buf,
              "INVITE sip:user@company.com SIP/2.0\r\n"
              "To: sip:j_user@company.com\r\n"
              "From: sip:caller@university.edu\r\n"
              "Call-ID: %d@10.1.1.1\r\n"
              "CSeq: 1 INVITE\r\n"
              "Via: SIP/2.0/UDP 10.1.1.2\r\n", count++ );
      msgLen[0] = strlen(buf);
      if (setting.genReqPort)
      {
         sendMessage( fd, msgVec, msgLen, 1, setting.genReqIp, setting.genReqPort );
         stats.numGenReq++;
      }
    
      sprintf(buf,"200\r\n"
              "Via: SIP/2.0/UDP lg.cathaynetworks.com:5061;branch=%x.%hx\r\n"
              "Via: SIP/2.0/UDP 10.1.1.2\r\n", 
              setting.genRespDstIp, setting.genRespDstPort);
      msgLen[0] = strlen(buf);
      if ( setting.genRespPort )
      {
         sendMessage( fd, msgVec, msgLen, 1, setting.genRespIp, setting.genRespPort );
         stats.numGenResp++;
      }
   }
}


// Local Variables:
// mode:c++
// c-file-style:"bsd"
// c-file-offsets:((case-label . +))
// c-basic-offset:4
// indent-tabs-mode:nil
// End:
