// Copyright (c) 2002 Cathay Networks, Inc.  
// see LICENSE file for details of VOCAL license

#ifndef peer_h
#define peer_h

/**************** Manage Peer connection *************************/
#include "network.h"

/// Data structure for sending updates from one proxy to another 
typedef struct
{
      // note if you change this data structure, you probably need to change the type 
      // so as to remain interoperable to peer with exiting versions 
      Int8 type;
      Int8 size;
      Int16 port;
      Int32 hash;
      Int32 ip;
      struct
      {
            Int32 low;
            Int32 high;
      } time;
} Update;

/// data struct for keeping track of all peers we talk too 
typedef struct 
{
      Int64 id; // formed by 32 bit IP addr in high and port low 32 bits
      char peerName[512];
      Socket  fd;
      Int64 lastConnectTime; // last time we tried to connect 
      bool synced; // have sent all records to other side 
      int syncedRecords; // index of last record that was synced 
} Peer;

const int maxNumPeers = 1024;
extern Peer peer[maxNumPeers];
extern int numPeers;


/// Shutdown the connetion to a peer 
void closePeer( int p );

/// send an update of a single element in the rib to a particular peer
void sendUpdate( int ribIndex, int peerIndex );

/// get an update from a peer and flood it to other peers 
void receiveUpdate( Socket fd );

/// tries to connect to all the peers that we currently don't have a connection too
void processPeers();

/// a new peer connected to us, manage it 
void addPeer( Socket peerListenFd );

// Local Variables:
// mode:c++
// c-file-style:"bsd"
// c-file-offsets:((case-label . +))
// c-basic-offset:4
// indent-tabs-mode:nil
// End:
#endif
