// Copyright (c) 2002 Cathay Networks, Inc.  
// see LICENSE file for details of VOCAL license

/********************* Deal with monitoring downstream proxies ***/
#include <cassert>
#include <cstdio>
#include <cstring>
#include <errno.h>
#include <iostream.h>

#ifdef WIN32

#include <winsock2.h>
#include <stdlib.h>
#include <io.h>

#else

#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>

#endif


#include "time.h"
#include "config.h"
#include "udp.h"
#include "probe.h"
#include "stats.h"

int numDstProxies = 0;
DstProxy dstProxy[maxNumProxies];

/// send a probe to this proxy
void
sendProbe( int proxyIndex, Socket probeFd )
{
   stats.probesSent++;
    
   DstProxy& p=dstProxy[proxyIndex];
    
   if ( p.fd == INVALID_SOCKET )
   {
      // need to create a bound UDP socket to this proxy 
      p.fd = openSendPort( p.ip, p.port);
   }
    
   char buf[udpMaxMessageLength];
    
   // TODO fix this options message 
   sprintf(buf,
           "OPTIONS SIP/2.0/UDP \r\nVia: SIP/2.0/UDP %s:%d;branch=%d.%d\r\n",
           setting.proxyName, 
           setting.probePort,
           proxyIndex, 
           p.probeNum );
    
   char* msgVec[1];
   int msgLen[1];
   msgVec[0] = buf;
   msgLen[0] = strlen(buf);
    
   bool ok;
   if ( p.fd != INVALID_SOCKET )
   {
      // send on bound UDP fd 
      ok = sendMessage( p.fd, msgVec, msgLen, 1);
   }
   else
   {
      // send on a unbound fd 
      ok = sendMessage( probeFd, msgVec, msgLen, 1, p.ip, p.port );
   }
   Int64 now = getTime();
    
   if (ok==false)
   {
      if ( p.up )
      {
         p.up = false;
         cerr << "Proxy " << proxyIndex << " at " << p.ip << " went DOWN (ICMP error)"<< endl;
      }
   }
    
   p.lastProbeSendTime = now;
   p.probeNum++;
}


/// send a ping to down stream proxies and see if they are alive 
void
processProbes(int probeFd)
{
   // TODO - make the probe retry times configurable 
    
   Int64 now = getTime();
    
   for( int d=0; d<numDstProxies; d++)
   {
      DstProxy& p = dstProxy[d];
        
      if ( p.probe == false )
      {
         // ignore this one 
         continue;
      }
	
      // if we have received a probe in last 5 seconds, ignore this one 
      if ( p.lastProbeRecvTime + 5*oneSecond >= now )
      {
         continue;
      }
        
      // if it has been more than 6 seconds, call it dead 
      if ( p.lastProbeRecvTime + 6*oneSecond < now )
      {
         if ( p.up )
         {
            p.up = false;
            cerr << "Proxy " << d << " at " << p.ip << " went DOWN" << endl;
         }
            
         // every 5 seconds send it a probe to see if it is up 
         if ( p.lastProbeSendTime + 5*oneSecond < now )
         {
            sendProbe(d,probeFd);
         }
            
         continue;
      }
        
      // send it a probe every 200 ms 
      if ( p.lastProbeSendTime + oneSecond/5 <= now )
      {
         sendProbe(d,probeFd);
      }
   }
}


/// got a response from a downstream proxy - mark it as alive 
void
handleProbe(int fd)
{
   stats.probesReceived++;
    
   assert( fd != INVALID_SOCKET );
   //clog << "In handleProbe" << endl;
    
   char buf[udpMaxMessageLength];
   int len;
   unsigned int srcIp;
   unsigned short srcPort;
    
   if ( getMessage(fd,buf,&len,&srcIp,&srcPort) )    
   {
      Int64 now = getTime();
        
      //clog << "Received probe " << len << " bytes of " << buf 
      //     << "from 0x" << hex << srcIp << dec << " port " << srcPort <<endl;
        
      bool found = false;
      for ( int d=0; d<numDstProxies; d++ )
      {
         if ( dstProxy[d].ip == srcIp )
         {
            dstProxy[d].lastProbeRecvTime = now;
                
            if ( found )
            {
               cerr << "More than one proxy at host " << dstProxy[d].ip  << endl;
            }
                
            found = true;
            if ( dstProxy[d].up == false )
            {
               dstProxy[d].up = true;
               cerr << "Changed proxy " << d << " from down to up " << endl;
            }
         }
      }
        
      if ( !found )
      {
         cerr << "Recieved probe from host with no known proxies" << srcIp << endl;
      }
   }   
}

// Local Variables:
// mode:c++
// c-file-style:"bsd"
// c-file-offsets:((case-label . +))
// c-basic-offset:4
// indent-tabs-mode:nil
// End:
