// Copyright (c) 2002 Cathay Networks, Inc.  
// see LICENSE file for details of VOCAL license

#ifndef proxy_h
#define proxy_h
/********************* Deal with monitoring downstream proxies ***/

#include "network.h"

/// datastruct to keep track of all proxies this one connects too 
typedef struct 
{
      Socket fd; // file descriptor of socket connected to this proxy 
      unsigned int ip; // proxies ip address 
      int port; // port to send messages to
      int priority; // dns style priority of when to use this proxy 
      int weight;  // dns style weight of when to use this proxy 
      bool up; // is this proxy up or down 
      bool probe; // if this proxie should be probed or not 
      Int64 lastProbeSendTime; // time sent last probe message 
      Int64 lastProbeRecvTime; // time received last probe response 
      int probeNum; // counter of numProbes ever sent to this proxy 
} DstProxy;

const int maxNumProxies=1024;
extern DstProxy dstProxy[maxNumProxies];
extern int numDstProxies;


/// send a probe to this proxy
void sendProbe( int proxyIndex, Socket probeFd );

/// send a ping to down stream proxies and see if they are alive 
void processProbes(int probeFd);

/// got a response from a downstream proxy - mark it as alive 
void handleProbe(int fd);

// Local Variables:
// mode:c++
// c-file-style:"bsd"
// c-file-offsets:((case-label . +))
// c-basic-offset:4
// indent-tabs-mode:nil
// End:
#endif
