// Copyright (c) 2002 Cathay Networks, Inc.  
// see LICENSE file for details of VOCAL license

#include <cstdio>
#include <iostream.h>

#include "rib.h"
#include "time.h"
#include "config.h"
#include "peer.h"
#include "probe.h"

/****************** RIB manipulation *****************************/

Rib* rib=NULL;
int ribSize=0;

int
getRibIndex( unsigned int hash )
{
   int ret = hash%ribSize;
   if ( rib[ret].hash != hash )
   {
      rib[ret].hash = hash;
   }
   return ret;
}


void
expireRib( int index )
{
   if ( rib[index].proxy != -1 ) // check if allready expired 
   {
      Int64 now = getTime();
      if ( now >= rib[index].time + oneSecond * setting.stickyExpireTime  )
      {
         clog << "Expired " << index << endl;

         rib[index].proxy = -1;
         rib[index].time = 0;
         rib[index].hash = 0;
      }
   }
}


///
int 
getRibProxy( unsigned int hash )
{
   int i = getRibIndex(hash);
    
   if ( rib[i].proxy == -1 )
   {
      return -1;
   }
    
   expireRib(i);
    
   return rib[i].proxy;
}


/// return index of peer with this fd 
int
findPeer( Socket fd )
{
   for ( int p=0; p<numPeers; p++ )
   {
      if ( peer[p].fd == fd )
      {
         return p;
      }
   }
   return -1;
}


/// return index of proxy with this ip and port 
int
findProxy( unsigned int ip, int port )
{
   for ( int i=0; i<numDstProxies; i++ )
   {
      if ( (dstProxy[i].ip == ip) && (dstProxy[i].port==port) )
      {
         return i;
      }
   }
   return -1;
}


/// add a new proxy to the list of dstProxies 
void
addProxy( int ip, int port, int priority, int weight, bool probe )
{
   if ( findProxy(ip,port) != -1 )
   {
      cerr << "Tried to add duplicate proxy" << endl;
      return;
   }
    
   int p = numDstProxies++;
   dstProxy[p].fd = INVALID_SOCKET;
   dstProxy[p].ip = ip;
   dstProxy[p].port = port;
   dstProxy[p].probe = probe;
   dstProxy[p].up = (probe)?false:true;
   dstProxy[p].probeNum = 0;
   dstProxy[p].priority = priority;
   dstProxy[p].weight = weight;
}

// Local Variables:
// mode:c++
// c-file-style:"bsd"
// c-file-offsets:((case-label . +))
// c-basic-offset:4
// indent-tabs-mode:nil
// End:
