// Copyright (c) 2002 Cathay Networks, Inc.  
// see LICENSE file for details of VOCAL license

/********************* deal with statistics ***********************/
#include <cassert>
#include <iostream.h>

#include "network.h"
#include "stats.h"
#include "time.h"
#include "config.h"
#include "peer.h"
#include "rib.h"
#include "probe.h"

Stats stats;

///
void
processStats(bool force)
{
   static Int64 lastTime = 0;
   Int64 now = getTime();
   if ( lastTime + setting.printStatsTime*oneSecond > now )
   {
      if ( force == false )
      {
         return;
      }
   }
   return;
   
   Int64 elapse = now - lastTime;
   lastTime=now;
    

   cerr << "-------------------------------------------------" << endl;
    
   // state of peers 
   cerr << "Peers:" << endl;
   for( int p=0; p<numPeers; p++)
   {
      cerr << "\tPeer " << p ;
      if ( peer[p].peerName[0] != '\0' )
      {
         cerr << " " << peer[p].peerName;
      }
      cerr << " is ";
      if (peer[p].fd!=INVALID_SOCKET)
      {
         cerr << "up  ";
            
         if ( peer[p].synced )
         {
            cerr << " and synced";
         }
         else
         {
            cerr << " and syncronizing " << peer[p].syncedRecords << " of " << ribSize;
         }   
      }
      else
      {
         cerr << "down";
      }
      cerr << endl;
   }
    
   // state of proxies
   cerr << "Proxies:" << endl;
   for ( int d=0; d<numDstProxies; d++ )
   {
      cerr << "\tProxy " << d 
           << " is " << static_cast<const char*>(dstProxy[d].up?"up  ":"down")
           << " probe count = " << dstProxy[d].probeNum 
           << " addr=" << dstProxy[d].ip <<":"<< dstProxy[d].port
           << " priority=" << dstProxy[d].priority 
           << " weight=" << dstProxy[d].weight
           << endl;
   }
    
   // call distribution info 
   cerr << "Distribution:" << endl;
   int count[maxNumProxies];
   int total=0;
   for ( int pr=0; pr<numDstProxies; pr++) 
   {
      count[pr]=0;
   }

   for ( int i=0; i<ribSize; i++ )
   {
      expireRib(i);
      if ( rib[i].proxy != -1 )
      {
         assert( rib[i].proxy < numDstProxies );
         count[rib[i].proxy]++;
         total++;
      }
   }
   for (int proxy=0;proxy<numDstProxies;proxy++)
   {
      int percent = (total==0)?0:(count[proxy]*100)/total;
      cerr << "\tProxy " << proxy 
           << " " << count[proxy] 
           << " active " << percent << "%" << endl;
   }
    
   // rib view 
   if ( setting.verbose )
   {
      cerr << "Rib:" << endl;
      for ( int i=0; i<ribSize; i++ )
      {
         expireRib(i);
         if ( rib[i].proxy != -1 )
         {
            cerr << "\tHash " << rib[i].hash << "->" << rib[i].proxy << endl;
         }
      }
   }

   static int prevNumRequests=0;
   Int64 rate = (stats.numRequests-prevNumRequests);
   rate *= oneSecond;
   rate = rate/elapse;
   int irate = low32(rate);
   cerr << "Reqests: " << stats.numRequests << "( " << irate << " cps )" << endl;
   prevNumRequests = stats.numRequests;
    
   cerr << "numResponses: " << stats.numResponses << endl;
   cerr << "updatesSent: " << stats.updatesSent << endl;
   cerr << "updatesReceived: " << stats.updatesReceived << endl;
   cerr << "numGenReq: " << stats.numGenReq << endl;
   cerr << "numGenResp: " << stats.numGenResp << endl;
   cerr << "numSinkMsg: " << stats.numSinkMsg << endl;
   cerr << "numSyncConflicts: " << stats.numSyncConflicts << endl;
   cerr << "peersAdded: " << stats.peersAdded << endl;
   cerr << "numPeerCloses: " << stats.numPeerCloses << endl;
    
   int sent = stats.numGenReq+stats.numGenResp;
   int lost = stats.numGenReq+stats.numGenResp-stats.numSinkMsg;

   int lostPercent = (sent==0)?0:lost*100/sent;
   cerr << "genLost: " << lost << "(" << lostPercent << "%)" <<  endl;
    
   cerr << endl;
}

// Local Variables:
// mode:c++
// c-file-style:"bsd"
// c-file-offsets:((case-label . +))
// c-basic-offset:4
// indent-tabs-mode:nil
// End:
